import 'package:flutter_test/flutter_test.dart';
import 'package:insta_html_parser/insta_html_parser.dart';

void main() {

  test('Get posts urls from public profile', () async {
    List<String> _posts = await InstaParser.postsUrlsFromProfile('https://www.instagram.com/contreirasgustavo/');
    expect(_posts != null, true);
    for (int i = 0; i < 12; i++) {
      expect(_posts[i] != null, true);
      print('#${i+1} ' + _posts[i]);
      expect(_posts[i].length, 'https://www.instagram.com/p/BQQrPauBgvn'.length);
    }
    expect(_posts.length, 12);
  });

  test('Get posts urls from static field after parsing public profile', () async {
    await InstaParser.postsUrlsFromProfile('https://www.instagram.com/contreirasgustavo/');
    List<String> _posts = InstaParser.lastPostsParsed;
    expect(_posts != null, true);
    for (int i = 0; i < 12; i++) {
      expect(_posts[i] != null, true);
      print('#${i+1} ' + _posts[i]);
      expect(_posts[i].length, 'https://www.instagram.com/p/BQQrPauBgvn'.length);
    }
    expect(_posts.length, 12);
  });

  test('Get user data from public profile', () async {
    Map<String, String> _userData = await InstaParser.userDataFromProfile('https://www.instagram.com/contreirasgustavo/');
    expect(_userData != null, true);
    print(_userData.toString());
    expect(_userData.keys.length, 10);
  });

  test('Get user data from static field after parsing public profile', () async {
    await InstaParser.userDataFromProfile('https://www.instagram.com/contreirasgustavo/');
    Map<String, String> _userData = InstaParser.lastUserDataParsed;
    expect(_userData != null, true);
    print(_userData.toString());
    expect(_userData.keys.length, 10);
  });

  test('Get photo urls from a post that contains a photo', () async {
    Map<String, String> _photoUrls = await InstaParser.photoUrlsFromPost('https://www.instagram.com/p/BQQrPauBgvn/');
    expect(_photoUrls != null, true);
    expect(_photoUrls['small'] != null, true);
    expect(_photoUrls['medium'] != null, true);
    expect(_photoUrls['large'] != null, true);
    print('#1 ' + _photoUrls['small']);
    print('#2 ' + _photoUrls['medium']);
    print('#3 ' + _photoUrls['large']);
    expect(_photoUrls['small'].length, 'https://www.instagram.com/p/BQQrPauBgvn/media/?size=t'.length);
    expect(_photoUrls['small'].contains('size=t'), true);
    expect(_photoUrls['medium'].length, 'https://www.instagram.com/p/BQQrPauBgvn/media/?size=m'.length);
    expect(_photoUrls['medium'].contains('size=m'), true);
    expect(_photoUrls['large'].length, 'https://www.instagram.com/p/BQQrPauBgvn/media/?size=m'.length);
    expect(_photoUrls['large'].contains('size=l'), true);
  });

  test('Get photo urls from static field after parsing post with photo', () async {
    await InstaParser.photoUrlsFromPost('https://www.instagram.com/p/BQQrPauBgvn/');
    Map<String, String> _photoUrls = InstaParser.lastPhotoUrlsParsed;
    expect(_photoUrls != null, true);
    expect(_photoUrls['small'] != null, true);
    expect(_photoUrls['medium'] != null, true);
    expect(_photoUrls['large'] != null, true);
    print('#1 ' + _photoUrls['small']);
    print('#2 ' + _photoUrls['medium']);
    print('#3 ' + _photoUrls['large']);
    expect(_photoUrls['small'].length, 'https://www.instagram.com/p/BQQrPauBgvn/media/?size=t'.length);
    expect(_photoUrls['small'].contains('size=t'), true);
    expect(_photoUrls['medium'].length, 'https://www.instagram.com/p/BQQrPauBgvn/media/?size=m'.length);
    expect(_photoUrls['medium'].contains('size=m'), true);
    expect(_photoUrls['large'].length, 'https://www.instagram.com/p/BQQrPauBgvn/media/?size=m'.length);
    expect(_photoUrls['large'].contains('size=l'), true);
  });

  test('Get video url from a post that contains a video', () async {
    String _videoUrl = await InstaParser.videoUrlFromPost('https://www.instagram.com/p/BlglqvHjCFh/');
    expect(_videoUrl != null, true);
    print(_videoUrl);   
    expect(_videoUrl.length > 30, true);
    expect(_videoUrl.contains('.mp4'), true);
  });

  test('Get video url from static field after parsing post with video', () async {
    await InstaParser.videoUrlFromPost('https://www.instagram.com/p/BlglqvHjCFh/');
    String _videoUrl = InstaParser.lastVideoUrlParsed;
    expect(_videoUrl != null, true);
    print(_videoUrl);   
    expect(_videoUrl.length > 30, true);
    expect(_videoUrl.contains('.mp4'), true);
  });
  
}
